#include "hardware.h"
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>
#include <util/delay.h>
#include "timer.h"

volatile uchar timerTimeoutCnt;
volatile uchar timerLongTimeoutCnt;

/* ------------------------------------------------------------------------- */

void timerMsDelay(uchar ms)
{
    timerSetupTimeout(ms);
    while(!timerTimeoutOccurred());
}

/* ------------------------------------------------------------------------- */

void timerTicksDelay(uchar ticks)
{
char    d, until;

    cli();
#ifdef SFIOR
    SFIOR = _BV(PSR10);     /* reset prescaler */
#else
    GTCCR = _BV(PSRSYNC);
#endif
    until = TCNT0 + ticks;
    sei();
    do{
        d = TCNT0 - until;  /* enforce 8 bit arithmetics */
    }while(d < 0);
}

/* ------------------------------------------------------------------------- */

/* main configures Timer 0 with 1/64 prescaler
 * -> 12MHz/64 = 187.5kHz --> 5.333us
 * overflow every 256 counts = 732.422Hz --> 1.365ms
 */
ISR(TIMER0_OVF_vect, ISR_NOBLOCK)   /* run with global interrupt enable */
{
static uchar    prescaler = 1;

    cli();
    if(timerTimeoutCnt != 0)
        timerTimeoutCnt--;
    sei();  /* allow interrupts here */
    cli();
    if(--prescaler == 0){   /* scale down to 10 Hz */
        prescaler = (uchar)((double)F_CPU / 64 / 256 / 10 + 0.5);
        sei();
        if(timerLongTimeoutCnt != 0)
            timerLongTimeoutCnt--;
    }else{
        sei();
    }
}

void timerSetupTimeout(uchar msDuration) {
 msDuration++;   /* Our resolution is 1.4 ms. Add one unit to avoid almost zero delays for low values */
 timerTimeoutCnt = msDuration;
}
